'use strict'

import {
  CampaignModel,
  ShareModal,
  ProductInfoModel,
} from '../../schemas'
import nanoid from 'nanoid'

export const addCampaign = async (ctx, next) => {
  try {
    const {body} = ctx.request
    const {
      campaignName,
      campaignNickName,
      campaignUrl,
      campaignPid,
      productId,
      meta: {
        og_title,
        og_image,
        og_site_name,
        og_url,
        og_type,
        description,
        title,
      }
    } = body;

    /**
     * campaignRecordId 必须要有，以便于绑定活动信息
     */
    if (!campaignName || !campaignUrl || !campaignPid || !productId) {
      ctx.send(200, {
        code: 0,
        message: 'productId campaignName campaignUrl campaignPid must provide',
        data: null
      })
      return
    }

    let oldOne = await CampaignModel.findOne({campaignName: campaignName}).exec();

    if (oldOne) {
      ctx.send(200, {
        code: 0,
        message: 'campaignName already exist',
        data: null
      })
      return
    }
    oldOne = await CampaignModel.findOne({campaignPid: campaignPid}).exec();

    if (oldOne) {
      ctx.send(200, {
        code: 0,
        message: 'campaignPid already exist',
        data: null
      })
      return
    }

    const campaignEntity = await CampaignModel({
      campaignPid: campaignPid,
      campaignName: campaignName,
      campaignNickName: campaignNickName,
      campaignUrl: campaignUrl,
      productId: productId,
      campaignCode: nanoid(2),
      meta: {
        og_title: og_title,
        og_image: og_image,
        og_site_name: og_site_name,
        og_url: og_url,
        og_type: og_type,
        description: description,
        title: title,
      }
    });
    let result = await campaignEntity.save().catch(err => {
      ctx.throw(500, err);
    });

    ctx.body = {
      code: 1,
      data: result
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const removeCampaignByName = async (ctx, next) => {
  try {
    if (ctx.request.body && ctx.request.body.campaignName) {
      const campaignName = ctx.request.body.campaignName;
      const result = await CampaignModel.deleteOne({campaignName: campaignName}).catch(err => {
        ctx.throw(500, err);
      });
      ctx.body = {
        success: true,
        data: result
      }
    } else {
      ctx.status = 403
      ctx.body = {
        success: false,
        data: null
      }
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const removeCampaignByCode = async (ctx, next) => {
  try {

    if (ctx.request.body && ctx.request.body.campaignCode) {
      const campaignCode = ctx.request.body.campaignCode;
      const result = await CampaignModel.deleteOne({campaignCode: campaignCode}).catch(err => {
        ctx.throw(500, err);
      });
      ctx.body = {
        success: true,
        data: {data: result}
      }
    } else {
      ctx.status = 403
      ctx.body = {
        success: false,
        code: 0,
        data: null
      }
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getCampaignByPid = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.campaignPid) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }
    const {campaignPid} = query;
    const campaignInfo = await CampaignModel.findOne({campaignPid: campaignPid}).exec();
    if (!campaignInfo) {
      ctx.send(200, {
        code: 0,
        message: 'not fount by campaignPid',
        data: null
      })
      return
    }

    ctx.body = {
      code: 1,
      success: true,
      data: campaignInfo
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getCampaignByName = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.campaignName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {campaignName} = query;
    const campaignInfo = await CampaignModel.findOne({campaignName: campaignName}).exec();

    if (!campaignInfo) {
      ctx.send(200, {
        code: 0,
        message: 'not fount by campaignName',
        data: null
      })
      return
    }
    ctx.body = {
      code: 1,
      success: true,
      data: campaignInfo
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getCampaignByCampaignCode = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.campaignCode) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {campaignCode} = query
    const campaignInfo = await CampaignModel.findOne({campaignCode: campaignCode}).exec();

    if (!campaignInfo) {
      ctx.send(200, {
        code: 0,
        message: 'not fount by campaignCode',
        data: null
      })
      return
    }

    ctx.body = {
      code: 1,
      success: true,
      data: campaignInfo
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const upgradeCampaignByName = async (ctx, next) => {
  try {
    const {body} = ctx.request
    if (!body || !body.campaignName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }
    const {campaignName, campaign} = body;
    const campaignEntity = await CampaignModel.findOneAndUpdate({campaignName: campaignName}, campaign).exec();

    if (!campaignEntity) {
      ctx.send(200, {
        code: 0,
        message: 'not fount by campaignName',
        data: null
      })
      return
    }

    ctx.body = {
      code: 1,
      success: true,
      data: campaignEntity
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const upgradeCampaignByCode = async (ctx, next) => {
  try {
    const {body} = ctx.request
    if (!body || !body.campaignCode) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {campaignCode, campaign} = body
    const campaignEntity = await CampaignModel.findOneAndUpdate({campaignCode: campaignCode}, campaign).exec();

    if (!campaignEntity) {
      ctx.send(200, {
        code: 0,
        message: 'not fount by campaignCode',
        data: null
      })
      return
    }

    ctx.body = {
      success: true,
      code: 1,
      data: campaignEntity
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const addCampaignShare = async (ctx, next) => {
  try {
    const {body} = ctx.request
    const {
      campaignRecordId,
      shareName,
      areaCodes,
      sharePlatforms,
      shareOriginUrl,
      shareType,
      shareNumber,
      pageType,
      shareMetas,
      expriation
    } = body
    if (!campaignRecordId || !shareName || !shareMetas || !shareOriginUrl || !shareNumber || !shareType) {
      ctx.send(200, {
        code: 0,
        message: 'campaignRecordId || shareName || shareMetas || shareOriginUrl || shareNumber || shareType props must provide'
      })
      return;
    }

    const campaignResult = await CampaignModel.findOne({_id: campaignRecordId}).exec()
    if (!campaignResult) {
      ctx.send(200, {
        code: 0,
        message: 'not fount by campaignRecordId',
        data: null
      })
      return
    }

    const shareResult = await ShareModal.findOne({campaignRecordId: campaignRecordId, shareName: shareName}).exec()
    if (shareResult) {
      ctx.send(200, {
        code: 0,
        message: 'shareName already exist',
        data: null
      })
      return
    }

    const shareEntity = new ShareModal({
      campaignRecordId: campaignRecordId,
      shareName: shareName,
      areaCodes: areaCodes,
      sharePlatforms: sharePlatforms,
      shareOriginUrl: shareOriginUrl,
      shareType,
      shareNumber,
      pageType: pageType || '200',
      shareCode: nanoid(3),
      shareMetas: shareMetas,
      //   [
      //   {
      //     platform: 'facebook',
      //     areaCode: 'cn',
      //     meta: {
      //       og_title: 'facebook2',
      //       og_image: 'facebook2',
      //       og_site_name: 'facebook2',
      //       og_url: 'facebook2',
      //       og_type: 'facebook2',
      //       description: 'facebook2',
      //       title: 'facebook2',
      //     }
      //   }
      // ],
      expriation: expriation,
      // createUser: ctx.session().user,
    });
    const result = await shareEntity.save().catch(err => {
      ctx.throw(500, err);
    });
    ctx.body = {
      success: true,
      code: 1,
      data: result
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const removeShareByName = async (ctx, next) => {
  try {
    const {body} = ctx.request
    if (!body || !body.shareName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }
    const {shareName} = body
    const result = await ShareModal.deleteOne({shareName: shareName});
    ctx.body = {
      success: true,
      data: result
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const removeShareByCode = async (ctx, next) => {
  try {
    const {body} = ctx.request
    if (!body || !body.shareCode) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }
    const {shareCode} = body

    const result = await ShareModal.deleteOne({shareCode: shareCode});
    ctx.body = {
      success: true,
      data: result
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const upgradeShareByName = async (ctx, next) => {
  try {
    const {body} = ctx.request
    if (!body || !body.shareName || !body.campaignShare) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }
    const {shareName, campaignShare} = body

    const shareEntity = await ShareModal.findOneAndUpdate({shareName: shareName}, campaignShare).exec()
    ctx.body = {
      success: true,
      data: shareEntity
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const upgradeShareByCode = async (ctx, next) => {
  try {
    const {body} = ctx.request
    if (!body || !body.shareCode || !body.campaignShare) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {shareCode, campaignShare} = body

    const shareEntity = await ShareModal.findOneAndUpdate({shareCode: shareCode}, campaignShare).exec();
    ctx.body = {
      success: true,
      data: shareEntity
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getShareByName = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.shareName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {shareName} = query

    const result = await ShareModal.findOne({shareName: shareName}).exec();

    ctx.body = {
      success: true,
      data: result
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getShareByCode = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.shareCode) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {shareCode} = query

    const result = await ShareModal.findOne({shareCode: shareCode}).exec();

    ctx.body = {
      success: true,
      data: result
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getShareByContent = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.condition) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    let {condition} = query
    // const shareCode = ctx.query.shareCode;
    // const platform = ctx.query.platform;
    condition = JSON.parse(condition || '{}');

    let result = await ShareModal.findOne(...condition).exec();
    // result = await ShareModal.findOne({shareCode: shareCode, 'shareLinks.platform': platform}).exec();

    ctx.body = {
      success: true,
      data: result
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getShareByOriginLink = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.shareName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {shareOriginLink} = query
    const result = await ShareModal.findOne({shareOriginLink: shareOriginLink}).exec();

    ctx.body = {
      success: true,
      data: result
    }
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getShareByCampaignName = async (ctx, next) => {
  try {

    const {query} = ctx.request
    if (!query || !query.campaignName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {campaignName} = query;
    const result = await ShareModal.findOne({campaignName: campaignName}).exec();

    ctx.body = {
      success: true,
      data: result
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getShareLinkByPidAndNumber = async (ctx, next) => {
  try {
    const {query} = ctx.request
    if (!query || !query.campaignPid) { // || !query.shareNumber
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return;
    }

    const {campaignPid, shareNumber} = query;
    const campaignResult = await CampaignModel.findOne({campaignPid: campaignPid}).exec();
    if (!campaignResult) {
      ctx.send(200, {
        code: 0,
        message: 'campaign not exist '
      })
      return;
    }
    const {_id, campaignCode, productId} = campaignResult.toJSON()
    let shareResult
    if (shareNumber) {
      shareResult = await ShareModal.findOne({campaignRecordId: _id, shareNumber: shareNumber || '1000'}).exec();
    } else {
      shareResult = await ShareModal.findOne({campaignRecordId: _id}).exec();
    }

    if (!shareResult) {
      ctx.send(200, {
        code: 0,
        message: 'share not exist, maybe shareNumber error',
        data: {
          campaignRecordId: _id,
          shareNumber
        }
      })
      return;
    }
    const {shareCode} = shareResult.toJSON()

    let domainName = ''
    const domainSetResult = await ProductInfoModel.findOne({productId: productId}).exec();

    // if (!domainSetResult) {
    //   ctx.send(200, {
    //     code: 0,
    //     message: 'domainSet not exist, maybe productId error',
    //     data: {
    //       campaignRecordId: _id,
    //       shareNumber
    //     }
    //   })
    //   return;
    // }
    // const {domainSet} = domainSetResult.toJSON()
    // if (domainSet.length <= 0) {
    //   ctx.send(200, {
    //     code: 0,
    //     message: 'domainSet empty',
    //     data: {
    //       campaignRecordId: _id,
    //       shareNumber
    //     }
    //   })
    //   return;
    // }
    if (domainSetResult) {
      const {domainSet} = domainSetResult.toJSON()
      // 随机数 命中概率要重新计算一下
      domainName = domainSet[Math.round(Math.random() * (domainSet.length - 1))]
    }

    const words = "abcdefghijklmnopqrstuvwxyz0123456789".split("");
    let sharePath='sh'

    //混淆sh路径
    for (var i = 0; i < 2; i++) {
      var rand = words[Math.floor(Math.random() * words.length)]
      sharePath=sharePath+rand
    }

    let campaignCodePath=randomStringAdd(campaignCode,2)
    let shareCodePath=randomStringAdd(shareCode,2)

    console.log('sharePath---------',sharePath,campaignCode,campaignCodePath,shareCode,shareCodePath,)

    ctx.body = {
      success: true,
      code: 1,
      data: {
        shareLink: `http://${domainName}/${sharePath}/${campaignCodePath}/${shareCodePath}/`,
        sharePathLink: `/${sharePath}/${campaignCodePath}/${shareCodePath}/`,
        domainName,
        campaignCode,
        shareCode,
        campaignRecordId: _id,
        shareNumber
      }
    }

  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const addProduct = async (ctx, next) => {
  try {
    const {body} = ctx.request
    const {
      productId,
      productName,
      domainSet,
    } = body

    if (!productId || !productName || !domainSet) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return
    }

    const oldOne = await ProductInfoModel.findOne({
      productId,
      productName,
    }).exec()

    if (oldOne) {
      ctx.send(200, {
        code: 0,
        message: 'product already exist'
      })
      return
    }

    const result = await ProductInfoModel({
      productId,
      productName,
      domainSet,
    }).save().catch(err => {
      ctx.throw(500, err);
    });

    ctx.send(200, {
      status: 200,
      success: true,
      data: result
    })


  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const upgradeProductDomainSet = async (ctx, next) => {
  try {
    const {body} = ctx.request
    const {
      productId,
      productName,
      domainSet,
    } = body

    if ((!productId && !productName) || !domainSet) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return
    }

    const oldOne = await ProductInfoModel.findOne({
      productId,
      productName,
    }).exec()

    if (!oldOne) {
      ctx.send(200, {
        code: 0,
        message: 'productName not exist,upgrade fail'
      })
      return
    }

    const result = await ProductInfoModel.findOneAndUpdate({
      productId,
      productName,
    }, {
      domainSet,
    }).exec().catch(err => {
      ctx.throw(500, err);
    });

    ctx.send(200, {
      code: 1,
      success: true,
      data: result
    })


  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const removeProductDomainSet = async (ctx, next) => {
  try {
    const {body} = ctx.request
    const {
      productId,
      productName,
    } = body

    if (!productId && !productName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return
    }

    const result = await ProductInfoModel.deleteOne({
      productId,
      productName,
    }).catch(err => {
      ctx.throw(500, err);
    });

    if (!result) {
      ctx.send(200, {
        code: 0,
        message: 'productName not exist,remove fail'
      })
      return
    }

    ctx.send(200, {
      code: 1,
      success: true,
      data: result
    })


  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

export const getProductDomainSet = async (ctx, next) => {
  try {
    const {query} = ctx.request
    const {
      productId,
      productName,
    } = query

    if (!productId && !productName) {
      ctx.send(200, {
        code: 0,
        message: 'props must provide'
      })
      return
    }

    let result
    if (productId) {
      result = await ProductInfoModel.findOne({productId}).exec()
    } else {
      result = await ProductInfoModel.findOne({productName}).exec()
    }

    if (!result) {
      ctx.send(200, {
        code: 0,
        success: false,
        message: ' DomainSet not exist'
      })
      return
    }
    ctx.send(200, {
      code: 1,
      success: true,
      data: result
    })
  } catch (e) {
    ctx.send(500, {message: e.toString()})
  }
}

// export const upgradeShareMetaByShareName = async (ctx, next) => {
//
//   const shareName = ctx.query.shareName || '';
//
//   const shareInfo = await ShareModal.findOne({shareName: shareName}).exec();
//
//   ctx.body = {
//     success: true,
//     data: {shareInfo: shareInfo}
//   }
// }

function randomStringAdd (str, desc) {
  var words = "abcdefghijklmnopqrstuvwxyz0123456789".split("");
  var result = "";
  for (var i = 0; i < str.length; i += desc) {
      var rand = words[Math.floor(Math.random() * words.length)]
      result += str.substring(i, i + desc);
      if (i + desc <= str.length) {
          result += rand;
      }
  }
  return result;
}
